@@ -51,9 +51,18 @@ class RecordIndexer(object):
     """
 
     record_cls = Record
+    """Record class used for retriving and dumping records.
+
+    You can either subclass and overwrite this attribute, or provide the record
+    class to the constructor.
+    """
+
+    record_dumper = None
+    """Dumper instance to use with this record indexer."""
 
     def __init__(self, search_client=None, exchange=None, queue=None,
-                 routing_key=None, version_type=None, record_to_index=None):
+                 routing_key=None, version_type=None, record_to_index=None,
+                 record_cls=None, record_dumper=None):
         """Initialize indexer.
 
         :param search_client: Elasticsearch client.
@@ -65,6 +74,12 @@ class RecordIndexer(object):
             (Default: ``external_gte``)
         :param record_to_index: Function to extract the index and doc_type
             from the record.
+        :param record_cls: Record class used for retriving and dumping records.
+            If the ``Record.enable_jsonref`` flag is False, new-style record
+            dumping will be used for creating the Elasticsearch source
+            document.
+        :param record_dumper: Dumper instance to use for dumping the record.
+            Only has an effect for new-style record dumping.
         """
         self.client = search_client or current_search_client
         self._exchange = exchange
@@ -73,6 +88,11 @@ class RecordIndexer(object):
         self._routing_key = routing_key
         self._version_type = version_type or 'external_gte'
 
+        if record_cls:
+            self.record_cls = record_cls
+        if record_dumper:
+            self.record_dumper = record_dumper
+
     def record_to_index(self, record):
         """Get index/doc_type given a record.
 
@@ -154,6 +174,15 @@ class RecordIndexer(object):
         index, doc_type = self.record_to_index(record)
         index, doc_type = self._prepare_index(index, doc_type)
 
+        # Pop version arguments for backward compatibility if they were
+        # explicit set to None in the function call.
+        if 'version' in kwargs and kwargs['version'] is None:
+            kwargs.pop('version', None)
+            kwargs.pop('version_type', None)
+        else:
+            kwargs.setdefault('version', record.revision_id)
+            kwargs.setdefault('version_type', self._version_type)
+
         return self.client.delete(
             id=str(record.id),
             index=index,
@@ -233,7 +262,7 @@ class RecordIndexer(object):
     def _bulk_op(self, record_id_iterator, op_type, index=None, doc_type=None):
         """Index record in Elasticsearch asynchronously.
 
-        :param record_id_iterator: Iterator that yields record UUIDs.
+        :param record_id_iterator: dIterator that yields record UUIDs.
         :param op_type: Indexing operation (one of ``index``, ``create``,
             ``delete`` or ``update``).
         :param index: The Elasticsearch index. (Default: ``None``)
@@ -275,17 +304,28 @@ class RecordIndexer(object):
         :param payload: Decoded message body.
         :returns: Dictionary defining an Elasticsearch bulk 'delete' action.
         """
+        kwargs = {}
         index, doc_type = payload.get('index'), payload.get('doc_type')
         if not (index and doc_type):
             record = self.record_cls.get_record(
                 payload['id'], with_deleted=True)
             index, doc_type = self.record_to_index(record)
+            kwargs['_version'] = record.revision_id
+            kwargs['_version_type'] = self._version_type
+        else:
+            # Allow version to be sent in the payload (but only use if we
+            # haven't loaded the record.
+            if 'version' in payload:
+                kwargs['_version'] = payload['version']
+                kwargs['_version_type'] = self._version_type
         index, doc_type = self._prepare_index(index, doc_type)
 
         return {
             '_op_type': 'delete',
             '_index': index,
+            '_type': doc_type,
             '_id': payload['id'],
+            **kwargs,
         }
 
     def _index_action(self, payload):
@@ -304,6 +344,7 @@ class RecordIndexer(object):
         action = {
             '_op_type': 'index',
             '_index': index,
+            '_type': doc_type,
             '_id': str(record.id),
             '_version': record.revision_id,
             '_version_type': self._version_type,
@@ -317,17 +358,36 @@ class RecordIndexer(object):
         """Prepare the index/doc_type before an operation."""
         return build_alias_name(index), doc_type
 
-    @staticmethod
-    def _prepare_record(record, index, doc_type, arguments=None, **kwargs):
+    def _prepare_record(self, record, index, doc_type, arguments=None,
+                        **kwargs):
         """Prepare record data for indexing.
 
+        Invenio-Records is evolving and preparing an Elasticsearch source
+        document is now a responsibility of the Record class. For backward
+        compatibility, we use the ``Record.enable_jsonref`` flag to control
+        if we use the new record dumpers feature from Invenio-Records. Set the
+        flag to ``False`` (disabling JSONRef replacement) to use the new
+        style record dumping.
+
         :param record: The record to prepare.
         :param index: The Elasticsearch index.
         :param doc_type: The Elasticsearch document type.
         :param arguments: The arguments to send to Elasticsearch upon indexing.
         :param **kwargs: Extra parameters.
-        :returns: The record metadata.
+        :returns: The Elasticsearch source document.
         """
+        # New-style record dumping - we use the Record.enable_jsonref flag on
+        # the Record to control if we use the new simplified dumping.
+        if not getattr(record, 'enable_jsonref', True):
+            # If dumper is None, dumps() will use the default configured dumper
+            # on the Record class.
+            return record.dumps(dumper=self.record_dumper)
+
+        # Old-style dumping - the old style will still if INDEXER_REPLACE_REFS
+        # is False use the Record.dumps(), however the default implementation
+        # is backward compatible for new-style records. Also, we're adding
+        # extra information into the record like _created and _updated
+        # afterwards, which the Record.dumps() have no control over.
         if current_app.config['INDEXER_REPLACE_REFS']:
             data = copy.deepcopy(record.replace_refs())
         else:
