@@ -105,7 +105,7 @@ def test_index_prefixing(base_app):
     default_doc_type = app.config['INDEXER_DEFAULT_DOC_TYPE']
 
     with app.app_context():
-        with patch('invenio_records.api.Record.validate'):
+        with patch('invenio_records.api._records_state.validate'):
             record = Record.create({'title': 'Test'})
             record2 = Record.create({
                 '$schema': '/records/authorities/authority-v1.0.0.json',
@@ -176,4 +176,6 @@ def test_index_prefixing(base_app):
                 id=str(record3.id),
                 index='test-' + default_index,
                 doc_type=default_doc_type if lt_es7 else '_doc',
+                version=record3.revision_id,
+                version_type='external_gte',
             )
